from fastapi import APIRouter, Request
from pathlib import Path
import importlib
import inspect
import sys
from typing import Dict, Any, Union
import logging
from .config_manager import ConfigManager

logger = logging.getLogger(__name__)

class ArcCmsRouter:
    @staticmethod
    def _get_route_params(module: Any) -> Dict[str, Any]:
        """Extrait les paramètres de la fonction handler"""
        handler = None
        if hasattr(module, 'get'):
            handler = module.get
        elif hasattr(module, 'default'):
            handler = module.default
        
        if not handler:
            return {}
        
        sig = inspect.signature(handler)
        return {
            name: param.default if param.default != inspect.Parameter.empty else ...
            for name, param in sig.parameters.items()
            if name not in ['request', 'self']
        }

    @staticmethod
    def _get_route_path(filepath: Path, base_path: Path) -> str:
        """Convertit le chemin du fichier en route"""
        try:
            relative_path = filepath.resolve().relative_to(base_path.resolve()).with_suffix('')
        except ValueError as e:
            logger.error(f"Path error: {e}")
            raise

        parts = []
        for part in relative_path.parts:
            if part == 'index':
                continue
            if part.startswith('_'):
                continue
            # Gestion spéciale pour les fichiers paramétrés
            if part.startswith('[') and part.endswith(']') and filepath.suffix == '.py':
                part = f"{{{part[1:-1]}}}"
            parts.append(part)
        
        return '/' + '/'.join(parts)

    @staticmethod
    def _get_base_url(config_data: dict) -> str:
        """Extrait le base-url de la configuration"""
        try:
            routing = config_data.get('routing', {})
            base_url = routing.get('base-url', '').strip()
            if base_url and not base_url.startswith('/'):
                base_url = f'/{base_url}'
            logger.debug(f"Extracted base-url: {base_url}")  # Ajout pour debug
            return base_url
        except Exception as e:
            logger.error(f"Error extracting base-url: {e}")
            return ''

    @classmethod
    def register_routes(cls, router: APIRouter, base_path: Union[Path, str], is_global: bool = True):
        is_global = is_global if type(is_global) == bool else True
        """Enregistre toutes les routes"""
        if isinstance(base_path, str):
            base_path = Path(base_path)
        base_path = base_path.resolve()

        config = ConfigManager()
        modules_metadata = config.get_modules_metadata()

        # Debug: Affiche les modules chargés
        logger.debug(f"Loaded modules: {list(modules_metadata.keys())}")
        
        # Base URL global
        global_config = config._configs.get('global', {})
        global_base_url = cls._get_base_url(global_config)
        print(f"[pyarccore -> router.py] ArcCmsRouter | register_routes - global_base_url: '{global_base_url}'")

        # Ajout temporaire au PYTHONPATH
        parent_dir = str(base_path)
        original_sys_path = sys.path.copy()
        if parent_dir not in sys.path:
            sys.path.insert(0, parent_dir)

        try:
            # if is_global == True:
            # Routes globales
            views_path = base_path / 'views'
            for filepath in views_path.glob('**/*.py'):
                if filepath.name.startswith(('_', '.')):
                    continue
# 
                try:
                    route_path = cls._get_route_path(filepath, views_path)
                    full_path = f"{global_base_url}{route_path}" if global_base_url else route_path
                    cls._register_route(router, views_path, filepath, full_path)
                except Exception as e:
                    logger.error(f"Error loading route {filepath}: {e}")
            # else:
            # Routes des modules (seulement ceux valides)
            views_dir = base_path / "modules"
            modules_dir = views_dir
            if modules_dir.exists():
                for module_name in modules_metadata.keys():
                    module_config = config._configs.get(module_name, {})
                    module_base_url = cls._get_base_url(module_config)
                    if not(
                        type(module_base_url) == str and
                        len(module_base_url) > 0
                    ):
                        module_base_url = module_name
                    print(f"[pyarccore -> router.py] ArcCmsRouter | register_routes - pour le module: '{module_name}' a pour module_base_url:: ", module_base_url, "")
                    
                    module_path = modules_dir / f"{module_name}/views"
                    print(f"[pyarccore -> router.py] ArcCmsRouter | register_routes - pour le module: '{module_name}' a pour module_path:: ", module_path, "")
                    
                    for filepath in module_path.glob('**/*.py'):
                        if filepath.name.startswith(('_', '.')):
                            continue

#                         # print(f"[pyarccore -> router.py] ArcCmsRouter | register_routes - pour le module: '{module_name}', filepath:: ", filepath, "")
                        try:
                            
                            route_path = cls._get_route_path(filepath.parent, views_dir)
                            route_path = "".join(route_path.split(f"{module_name}/views/"))
                            print(f"[pyarccore -> router.py] ArcCmsRouter | register_routes - pour le module: '{module_name}' a pour route_path:: ", route_path, "")
                            # Construction du chemin complet selon les règles spécifiées
                            full_path = cls._build_module_route_path(
                                global_base_url,
                                module_name,
                                module_base_url,
                                route_path
                            )
                            cls._register_route(router, views_dir, filepath, full_path)
                        except Exception as e:
                            logger.error(f"Error loading route {filepath}: {e}")

        finally:
            sys.path = original_sys_path

    @classmethod
    def _build_module_route_path(
        cls,
        global_base_url: str,
        module_name: str,
        module_base_url: str,
        route_path: str
    ) -> str:
        """Construit le chemin complet selon les règles spécifiées"""
        parts = []
        
        # 1. Ajouter le base-url global s'il existe
        if global_base_url:
            parts.append(global_base_url)
        
        # 2. Gestion du module
        if module_base_url:
            # Si module_base_url est défini, on l'utilise
            parts.append(module_base_url)
        elif global_base_url:
            # Sinon on ajoute le nom du module seulement si global_base_url est défini
            parts.append(f"/{module_name}")
        
        # 3. Ajouter le chemin de la route
        parts.append(route_path)
        
        # Joindre les parties et nettoyer les doubles slashes
        full_path = ''.join(parts).replace('//', '/')
        
        # S'assurer que le chemin commence par un slash
        if not full_path.startswith('/'):
            full_path = f'/{full_path}'
        
        return full_path

    @classmethod
    def _register_route(cls, router: APIRouter, base_path: Path, filepath: Path, route_path: str):
        """Enregistre une route individuelle"""
        module_path = '.'.join(filepath.relative_to(base_path.parent).with_suffix('').parts)

        module = importlib.import_module(module_path)
        
        # Enregistrement des méthodes HTTP
        for method in ['get', 'post', 'put', 'delete', 'patch']:
            if hasattr(module, method):
                handler = getattr(module, method)
                getattr(router, method)(
                    route_path,
                    **({'response_model': handler.__annotations__.get('return')} 
                    if hasattr(handler, '__annotations__') else {})
                )(handler)
        
        # Fallback pour default
        if hasattr(module, 'default') and not any(hasattr(module, m) for m in ['get', 'post', 'put', 'delete', 'patch']):
            handler = module.default
            router.get(route_path)(handler)

        logger.info(f"Route registered: {route_path} -> {module_path}")


import xml.etree.ElementTree as ET
from pathlib import Path
import threading
import hashlib
from functools import lru_cache
from typing import Dict, Any, Optional, List
import logging
from .versionning import check_version
from pkg_resources import get_distribution, DistributionNotFound
import subprocess
import sys
import re


def element_to_dict(element):
    """Convertit un élément XML et ses enfants en dictionnaire."""
    result = {}
    for child in element:
        # Si l'élément a des enfants, on traite récursivement
        if len(child) > 0:
            child_data = element_to_dict(child)
        else:
            # Sinon on prend simplement son texte
            child_data = child.text
        
        # Gestion des éléments multiples avec le même nom
        if child.tag in result:
            # Si la clé existe déjà, on convertit en liste
            if isinstance(result[child.tag], list):
                result[child.tag].append(child_data)
            else:
                result[child.tag] = [result[child.tag], child_data]
        else:
            result[child.tag] = child_data
    
    # Si l'élément a des attributs, on les ajoute
    if element.attrib:
        result.update(element.attrib)
    
    return result

logger = logging.getLogger(__name__)

class ConfigManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._configs = {}
        self._lock = threading.RLock()
        self._project_root = None
        self._file_hashes = {}
        self._modules_metadata = {}

    def set_project_root(self, path: Path):
        """Définit la racine du projet"""
        with self._lock:
            self._project_root = path.resolve()

    def load_all(self):
        """Charge toutes les configurations"""
        if self._project_root is None:
            logger.error("Project root not set, call set_project_root() first")
            return

        # Global
        global_config = self._project_root / 'config.xml'
        if global_config.exists():
            self._load_config('global', global_config)

        # Modules
        modules_dir = self._project_root / 'modules'
        if modules_dir.exists():
            for module_path in modules_dir.iterdir():
                if module_path.is_dir():
                    config_file = module_path / 'config.xml'
                    if config_file.exists():
                        if self._validate_module_structure(config_file):
                            self._load_config(module_path.name, config_file)
                            self._load_module_metadata(module_path.name, config_file)
                            self._check_module_requirements(module_path.name)
                            self._install_module_plugins(module_path.name)
                        else:
                            logger.warning(f"Module {module_path.name} has invalid structure and will be ignored")

        # Install global plugins after modules are loaded
        self._install_global_plugins()

    def _validate_module_structure(self, config_file: Path) -> bool:
        """Valide la structure minimale du module"""
        try:
            tree = ET.parse(config_file)
            root = tree.getroot()
            
            # Vérification des métadonnées minimales
            metadatas = root.find('metadatas')
            if metadatas is None:
                return False
                
            required_fields = ['id', 'name', 'version']
            for field in required_fields:
                if metadatas.find(field) is None:
                    return False
                    
            return True
        except ET.ParseError:
            return False

    def _load_module_metadata(self, module_name: str, config_file: Path):
        """Charge les métadonnées du module"""
        try:
            # print(f"[pyarccore -> config_manager.py] ConfigManager | _load_module_metadata - module_name:: ", module_name)
            # print(f"[pyarccore -> config_manager.py] ConfigManager | _load_module_metadata - config_file:: ", config_file)
            tree = ET.parse(config_file)
            root = tree.getroot()
            metadatas = root.find('metadatas')
            # print(f"[pyarccore -> config_manager.py] ConfigManager | _load_module_metadata - metadatas:: ", element_to_dict(metadatas))
            
            if metadatas is None:
                return
            # print(f"[pyarccore -> config_manager.py] ConfigManager | _load_module_metadata - metadatas is not None")
                
            metadata = {
                'id': metadatas.findtext('id', '').strip(),
                'name': metadatas.findtext('name', '').strip(),
                'description': metadatas.findtext('description', '').strip(),
                'version': metadatas.findtext('version', '').strip(),
                'authors': [],
                'requirements': [],
                'plugins': []
            }
            
            # Auteurs
            authors = metadatas.find('authors')
            if authors is not None:
                for author in authors.findall('author'):
                    metadata['authors'].append({
                        'name': author.findtext('name', '').strip(),
                        'email': author.findtext('email', '').strip(),
                        'position': author.findtext('position', '').strip(),
                        'company': author.findtext('company', '').strip()
                    })
            
            # Requirements
            requirements = metadatas.find('requirements')
            if requirements is not None:
                for req in requirements.findall('requirement'):
                    metadata['requirements'].append({
                        'id': req.findtext('id', '').strip(),
                        'version': req.findtext('version', '').strip()
                    })
            
            # Plugins
            plugins = metadatas.find('plugins')
            if plugins is not None:
                for plugin in plugins.findall('plugin'):
                    metadata['plugins'].append({
                        'id': plugin.findtext('id', '').strip(),
                        'version': plugin.findtext('version', '').strip()
                    })
            
            self._modules_metadata[module_name] = metadata
            
        except Exception as e:
            logger.error(f"Failed to load metadata for module {module_name}: {e}")

    def _check_module_requirements(self, module_name: str):
        """Vérifie strictement les dépendances entre modules"""
        if module_name not in self._modules_metadata:
            return
            
        module_meta = self._modules_metadata[module_name]
        requirements = module_meta.get('requirements', [])
        loaded_modules = self._modules_metadata
        
        # Crée un mapping ID -> (nom_module, metadata)
        id_to_module = {
            meta['id']: (name, meta)
            for name, meta in loaded_modules.items() 
            if 'id' in meta
        }
        
        missing_requirements = []
        version_mismatches = []
        
        for req in requirements:
            req_id = req['id']
            req_version = req.get('version', '').strip()
            
            if req_id in id_to_module:
                mod_name, mod_meta = id_to_module[req_id]
                
                if req_version:  # Si une version est spécifiée
                    if not check_version(mod_meta['version'], req_version):
                        version_mismatches.append(
                            f"{req_id} (required: {req_version}, found: {mod_meta['version']})"
                        )
                continue
                
            # Recherche dans le filesystem
            modules_dir = self._project_root / 'modules'
            found = False
            
            if modules_dir.exists():
                for mod_dir in modules_dir.iterdir():
                    config_file = mod_dir / 'config.xml'
                    if config_file.exists() and self._validate_module_structure(config_file):
                        try:
                            tree = ET.parse(config_file)
                            mod_id = tree.findtext('metadatas/id', '').strip()
                            
                            if mod_id == req_id:
                                # Charge le module manquant
                                self._load_config(mod_dir.name, config_file)
                                self._load_module_metadata(mod_dir.name, config_file)
                                loaded_modules.update(self._modules_metadata)
                                
                                # Vérifie la version après chargement
                                mod_meta = self._modules_metadata[mod_dir.name]
                                if req_version and not check_version(mod_meta['version'], req_version):
                                    version_mismatches.append(
                                        f"{req_id} (required: {req_version}, found: {mod_meta['version']})"
                                    )
                                found = True
                                break
                        except ET.ParseError:
                            continue
            
            if not found:
                missing_requirements.append(
                    f"{req_id} {f'(version {req_version})' if req_version else ''}"
                )
        
        # Gestion des erreurs
        if missing_requirements or version_mismatches:
            error_msg = []
            
            if missing_requirements:
                error_msg.append(f"Missing requirements: {', '.join(missing_requirements)}")
            
            if version_mismatches:
                error_msg.append(f"Version mismatches: {', '.join(version_mismatches)}")
            
            full_error = f"Module {module_name} requirements not satisfied:\n" + "\n".join(error_msg)
            logger.error(full_error)
            raise ValueError(full_error)

    def get_module_id_map(self) -> Dict[str, str]:
        """Retourne un mapping des IDs vers les noms de dossiers"""
        with self._lock:
            return {
                meta['id']: name 
                for name, meta in self._modules_metadata.items()
                if 'id' in meta
            }

    def _install_global_plugins(self):
        """Installe les plugins globaux de manière tolérante"""
        if 'global' not in self._configs or '_hash' not in self._configs['global']:
            return
            
        plugins = self._configs['global'].get('metadatas', {}).get('plugins', [])
        for plugin in plugins:
            try:
                self._install_plugin(plugin)
            except Exception as e:
                logger.error(f"Erreur lors du traitement du plugin global {plugin.get('id')}: {str(e)}")
                continue  # Continue avec les plugins suivants

    def _install_module_plugins(self, module_name: str):
        """Installe les plugins d'un module de manière tolérante"""
        if module_name not in self._modules_metadata:
            return
            
        plugins = self._modules_metadata[module_name].get('plugins', [])
        for plugin in plugins:
            try:
                self._install_plugin(plugin)
            except Exception as e:
                logger.error(f"Erreur lors du traitement du plugin {plugin.get('id')} pour {module_name}: {str(e)}")
                continue  # Continue avec les plugins suivants

    def _install_plugin(self, plugin: Dict[str, str]):
        """Installe un plugin Python de manière tolérante"""
        plugin_id = plugin.get('id', '').strip()
        if not plugin_id:
            return
            
        version_spec = plugin.get('version', '').strip()
        package_spec = plugin_id + (version_spec if version_spec.startswith(('==', '>=', '<=', '>', '<', '!=')) else f"=={version_spec}" if version_spec else "")
        
        try:
            # Vérification si déjà installé
            installed_dist = get_distribution(plugin_id)
            installed_version = installed_dist.version
            
            if version_spec and not check_version(installed_version, version_spec):
                logger.warning(
                    f"Plugin {plugin_id} version mismatch (installed: {installed_version}, required: {version_spec})\n"
                    f"Trying to install correct version..."
                )
                raise DistributionNotFound  # Force la réinstallation
            
            logger.info(f"Plugin {plugin_id} (version {installed_version}) satisfait les exigences")
            return
            
        except DistributionNotFound:
            # Tentative d'installation
            logger.info(f"Installation du plugin: {package_spec}")
            try:
                subprocess.check_call(
                    [sys.executable, "-m", "pip", "install", package_spec],
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL
                )
                logger.info(f"Plugin installé avec succès: {package_spec}")
            except subprocess.CalledProcessError:
                logger.error(f"Échec de l'installation du plugin: {package_spec}")
                # On continue malgré l'échec
            except Exception as e:
                logger.error(f"Erreur lors de l'installation de {plugin_id}: {str(e)}")
                # On continue malgré l'erreur


    def _load_config(self, name: str, path: Path):
        """Charge une configuration XML"""
        path = path.resolve()
        file_hash = self._file_hash(path)
        try:
            tree = ET.parse(path)
            config = self._xml_to_dict(tree.getroot())
            config['_hash'] = file_hash

            with self._lock:
                self._configs[name] = config
                # logger.debug(f"Config loaded for {name}: {config}")  # Affiche le contenu chargé

        except ET.ParseError as e:
            logger.error(f"XML parsing error in config {path}: {e}")
        except Exception as e:
            logger.error(f"Failed to load config {path}: {e}")

    def _file_hash(self, path: Path) -> str:
        """Calcule le hash d'un fichier"""
        path = path.resolve()
        if path not in self._file_hashes:
            try:
                with open(path, 'rb') as f:
                    self._file_hashes[path] = hashlib.md5(f.read()).hexdigest()
            except IOError as e:
                logger.error(f"Could not read file {path}: {e}")
                return ""
        return self._file_hashes[path]

    def _xml_to_dict(self, element) -> Dict[str, Any]:
        """Convertit XML en dict"""
        result = {**element.attrib}  # Conserve les attributs XML
        
        # Gestion spéciale pour les balises avec attributs et texte
        if element.text and element.text.strip():
            if not result:  # Si pas d'attributs, on prend directement le texte
                return element.text.strip()
            result['_text'] = element.text.strip()
        
        for child in element:
            child_data = self._xml_to_dict(child)
            if child.tag in result:
                if isinstance(result[child.tag], list):
                    result[child.tag].append(child_data)
                else:
                    result[child.tag] = [result[child.tag], child_data]
            else:
                result[child.tag] = child_data or child.text
        
        # Conversion spéciale pour les listes de protocoles
        if element.tag == 'protocoles':
            if 'protocole' in result and isinstance(result['protocole'], list):
                # Garde la liste telle quelle
                pass
            elif 'protocole' in result:
                # Convertit en liste si un seul protocole
                result['protocole'] = [result['protocole']]
        
        return result

    @lru_cache(maxsize=1024)
    def get(self, module: str, key: str, default: Any = None) -> Any:
        """Récupère une valeur avec cache"""
        with self._lock:
            keys = key.split('.')
            try:
                value = self._configs.get(module, {})
                for k in keys:
                    value = value[k]
                return value
            except (KeyError, TypeError):
                return default

    def get_project_root(self) -> Path:
        """Retourne la racine du projet"""
        with self._lock:
            return self._project_root

    def get_modules_metadata(self) -> Dict[str, Dict]:
        """Retourne les métadonnées de tous les modules"""
        with self._lock:
            return self._modules_metadata.copy()

    def get_files_from_directory(
        self, 
        directory_name: str, 
        module: str = None, 
        file_filter: str = None
    ) -> Dict[str, List[Path]]:
        """
        Récupère les fichiers d'un répertoire spécifique dans le projet
        
        Args:
            directory_name: Nom du répertoire ('public', 'pages', etc.)
            module: Nom du module (None pour global)
            file_filter: Regex pour filtrer les fichiers
            
        Returns:
            Dictionnaire des chemins des fichiers trouvés
        """
        result = {}
        with self._lock:
            if module is None:
                # Global directory
                dir_path = self._project_root / directory_name
                if dir_path.exists() and dir_path.is_dir():
                    files = self._filter_files(dir_path, file_filter)
                    if files:
                        result['global'] = files
            else:
                # Module directory
                if module in self._modules_metadata:
                    dir_path = self._project_root / 'modules' / module / directory_name
                    if dir_path.exists() and dir_path.is_dir():
                        files = self._filter_files(dir_path, file_filter)
                        if files:
                            result[module] = files
            
            return result

    def _filter_files(self, directory: Path, pattern: str = None) -> List[Path]:
        """Filtre les fichiers selon un motif regex"""
        files = []
        for item in directory.rglob('*'):
            if item.is_file():
                if pattern is None or re.match(pattern, item.name):
                    files.append(item)
        return files

    def get_all_files_from_directory(
        self,
        directory_name: str,
        file_filter: str = None,
        include_global: bool = True
    ) -> Dict[str, List[Path]]:
        """
        Récupère les fichiers d'un répertoire spécifique dans tous les modules
        
        Args:
            directory_name: Nom du répertoire ('public', 'pages', etc.)
            file_filter: Regex pour filtrer les fichiers
            include_global: Inclure les fichiers globaux si True
            
        Returns:
            Dictionnaire avec:
                - clé: nom du module ou 'global'
                - valeur: liste des fichiers trouvés
        """
        result = {}
        
        with self._lock:
            # Fichiers globaux
            if include_global:
                global_files = self.get_files_from_directory(directory_name, None, file_filter)
                if global_files.get('global'):
                    result['global'] = global_files['global']
            
            # Fichiers des modules
            for module_name in self._modules_metadata.keys():
                module_files = self.get_files_from_directory(directory_name, module_name, file_filter)
                if module_files.get(module_name):
                    result[module_name] = module_files[module_name]
        
        return result

    def get_config_params(
        self, 
        param_path: str, 
        module: str = None, 
        key_filter: str = None
    ) -> Dict[str, Any]:
        """
        Récupère des paramètres spécifiques de configuration
        
        Args:
            param_path: Chemin du paramètre ('metadatas.authors')
            module: Nom du module (None pour global)
            key_filter: Regex pour filtrer les clés
            
        Returns:
            Dictionnaire des paramètres trouvés
        """
        with self._lock:
            config = self._configs.get('global' if module is None else module, {})
            keys = param_path.split('.')
            
            try:
                value = config
                for k in keys:
                    value = value[k]
                
                if key_filter is None:
                    return value
                
                if isinstance(value, dict):
                    return {
                        k: v for k, v in value.items() 
                        if re.match(key_filter, k)
                    }
                elif isinstance(value, list):
                    return [
                        item for item in value 
                        if isinstance(item, dict) and 
                        any(re.match(key_filter, k) for k in item.keys())
                    ]
                return value
                
            except (KeyError, TypeError):
                return {}
    def get_all_config_params(
        self,
        param_path: str,
        key_filter: str = None,
        include_global: bool = True
    ) -> Dict[str, Dict[str, Any]]:
        """
        Récupère des paramètres spécifiques de configuration pour tous les modules
        
        Args:
            param_path: Chemin du paramètre ('metadatas.authors')
            key_filter: Regex pour filtrer les clés
            include_global: Inclure la configuration globale si True
            
        Returns:
            Dictionnaire avec:
                - clé: nom du module ou 'global'
                - valeur: paramètres trouvés pour ce module
        """
        result = {}
        
        with self._lock:
            # Configuration globale
            if include_global and 'global' in self._configs:
                global_params = self.get_config_params(param_path, 'global', key_filter)
                if global_params:
                    result['global'] = global_params
            
            # Configurations des modules
            for module_name in self._modules_metadata.keys():
                module_params = self.get_config_params(param_path, module_name, key_filter)
                if module_params:
                    result[module_name] = module_params
        
        return result
    def get_protocols(self) -> List[str]:
        """Retourne la liste des protocoles configurés"""
        with self._lock:
            try:
                protocoles = self.get('global', 'protocoles')
                if isinstance(protocoles, dict):
                    protocols = protocoles.get('protocole', [])
                    if isinstance(protocols, str):
                        protocols = [protocols]
                    return protocols
                return []
            except Exception:
                return []


from typing import Any, Dict, Optional
from fastapi import WebSocket
from .websocket_manager import WebSocketManager

async def emit(event: str, data: Any, namespace: str = '/', room: str = None):
    """Émet un événement WebSocket"""
    await WebSocketManager().emit(event, data, namespace, room)

async def broadcast(event: str, data: Any, namespace: str = None):
    """Diffuse un événement à tous les clients"""
    await WebSocketManager().broadcast(event, data, namespace)

def join_room(room: str, websocket: WebSocket, namespace: str = '/'):
    """Ajoute un client à une room"""
    WebSocketManager().join_room(room, websocket, namespace)

def leave_room(room: str, websocket: WebSocket, namespace: str = '/'):
    """Retire un client d'une room"""
    WebSocketManager().leave_room(room, websocket, namespace)

def get_connected_clients(namespace: str = '/') -> list:
    """Retourne les clients connectés"""
    return WebSocketManager().get_connected_clients(namespace)

def get_rooms() -> Dict[str, list]:
    """Retourne toutes les rooms"""
    return WebSocketManager().get_rooms()

def get_namespaces() -> list:
    """Retourne tous les namespaces"""
    return WebSocketManager().get_namespaces()


import asyncio
import importlib
import inspect
import json
import logging
import threading
from pathlib import Path
from typing import Any, Dict, List, Optional, Union, Callable, Coroutine
from fastapi import WebSocket, WebSocketDisconnect
from fastapi.websockets import WebSocketState
from .config_manager import ConfigManager

logger = logging.getLogger(__name__)

class WebSocketManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._project_root = None
        self._config = ConfigManager()
        self._connections: Dict[str, Dict[str, WebSocket]] = {}
        self._rooms: Dict[str, List[str]] = {}
        self._namespaces: Dict[str, Dict[str, Any]] = {}
        self._event_handlers: Dict[str, Dict[str, Callable]] = {}
        self._lock = threading.RLock()
        self._server = None

    def set_project_root(self, path: Path):
        """Définit la racine du projet"""
        with self._lock:
            self._project_root = path.resolve()

    async def initialize(self):
        """Initialise le serveur WebSocket et charge les handlers"""
        with self._lock:
            if not self._project_root:
                raise ValueError("Project root not set")

            # Charge les handlers globaux
            global_ws_dir = self._project_root / 'ws'
            if global_ws_dir.exists():
                self._load_handlers('global', global_ws_dir)

            # Charge les handlers des modules
            modules_metadata = self._config.get_modules_metadata()
            modules_dir = self._project_root / 'modules'
            if modules_dir.exists():
                for module_name in modules_metadata.keys():
                    module_ws_dir = modules_dir / module_name / 'ws'
                    if module_ws_dir.exists():
                        self._load_handlers(module_name, module_ws_dir)

    def _load_handlers(self, context: str, ws_dir: Path):
        """Charge les handlers WebSocket depuis un répertoire"""
        for handler_file in ws_dir.glob('**/*.py'):
            if handler_file.name.startswith(('_', '.')):
                continue

            try:
                # Construction du chemin d'import correct
                rel_path = handler_file.relative_to(self._project_root)
                module_path = '.'.join(rel_path.with_suffix('').parts)
                
                # Debug: Affiche le chemin d'import
                logger.debug(f"Importing WebSocket handler from: {module_path}")
                
                module = importlib.import_module(module_path)

                for name, obj in inspect.getmembers(module):
                    if inspect.isfunction(obj) or inspect.iscoroutinefunction(obj):
                        if name.startswith('on_'):
                            event_name = name[3:]
                            self._register_handler(context, event_name, obj)

            except Exception as e:
                logger.error(f"Failed to load WebSocket handler {handler_file}: {e}")
                logger.error(f"Import path attempted: {module_path}")
                logger.error(f"Project root: {self._project_root}")
                logger.error(f"Relative path: {rel_path}")

    def _register_handler(self, context: str, event_name: str, handler: Callable):
        """Enregistre un handler d'événement"""
        with self._lock:
            # Applique les préfixes du module
            module_config = self._config._configs.get(context, {})
            websocket_config = module_config.get('websocket', {})
            
            # Préfixe l'événement si configuré
            if 'event-prefix' in websocket_config:
                event_name = f"{websocket_config['event-prefix']}:{event_name}"
            
            # Enregistre avec le namespace approprié
            namespace = '/'
            if 'namespace-prefix' in websocket_config:
                namespace = websocket_config['namespace-prefix']
            elif 'module-prefix' in websocket_config:
                namespace = websocket_config['module-prefix']
            elif context != 'global':
                namespace = f"/{context}"

            if namespace not in self._event_handlers:
                self._event_handlers[namespace] = {}
            
            self._event_handlers[namespace][event_name] = handler
            logger.info(f"Registered WebSocket handler: {namespace}/{event_name}")

    async def handle_connection(self, websocket: WebSocket, namespace: str = '/'):
        """Gère une nouvelle connexion WebSocket"""
        await websocket.accept()
        connection_id = id(websocket)
        
        with self._lock:
            if namespace not in self._connections:
                self._connections[namespace] = {}
            self._connections[namespace][str(connection_id)] = websocket

        try:
            while True:
                data = await websocket.receive_text()
                await self._handle_message(websocket, namespace, data)
        except WebSocketDisconnect:
            await self._handle_disconnect(websocket, namespace)
        except Exception as e:
            logger.error(f"WebSocket error: {e}")
            await self._handle_disconnect(websocket, namespace)

    async def _handle_message(self, websocket: WebSocket, namespace: str, data: str):
        """Traite un message WebSocket"""
        try:
            message = json.loads(data)
            event = message.get('event')
            payload = message.get('data')
            
            if not event:
                return

            # Trouve le handler approprié
            handler = None
            with self._lock:
                if namespace in self._event_handlers and event in self._event_handlers[namespace]:
                    handler = self._event_handlers[namespace][event]

            if handler:
                try:
                    if inspect.iscoroutinefunction(handler):
                        await handler(websocket, payload)
                    else:
                        handler(websocket, payload)
                except Exception as e:
                    logger.error(f"Error in WebSocket handler {event}: {e}")
        except json.JSONDecodeError:
            logger.error("Invalid JSON received over WebSocket")

    async def _handle_disconnect(self, websocket: WebSocket, namespace: str):
        """Gère la déconnexion d'un client"""
        connection_id = id(websocket)
        
        with self._lock:
            if namespace in self._connections and str(connection_id) in self._connections[namespace]:
                del self._connections[namespace][str(connection_id)]

        # Quitte toutes les rooms
        for room in list(self._rooms.keys()):
            self.leave_room(room, websocket)

    async def emit(self, event: str, data: Any, namespace: str = '/', room: str = None):
        """Émet un événement à des clients spécifiques"""
        message = json.dumps({'event': event, 'data': data})
        
        with self._lock:
            if room:
                # Émission à une room spécifique
                if room in self._rooms:
                    for connection_id in self._rooms[room]:
                        for ns in [namespace, '/']:  # Essaye le namespace spécifié puis le global
                            if ns in self._connections and connection_id in self._connections[ns]:
                                try:
                                    websocket = self._connections[ns][connection_id]
                                    if websocket.client_state == WebSocketState.CONNECTED:
                                        await websocket.send_text(message)
                                except Exception as e:
                                    logger.error(f"Error emitting to {connection_id}: {e}")
            else:
                # Émission à tous les clients du namespace
                for ns in [namespace, '/']:  # Essaye le namespace spécifié puis le global
                    if ns in self._connections:
                        for connection_id, websocket in self._connections[ns].items():
                            try:
                                if websocket.client_state == WebSocketState.CONNECTED:
                                    await websocket.send_text(message)
                            except Exception as e:
                                logger.error(f"Error emitting to {connection_id}: {e}")

    def join_room(self, room: str, websocket: WebSocket, namespace: str = '/'):
        """Ajoute un client à une room"""
        connection_id = str(id(websocket))
        
        with self._lock:
            # Applique le préfixe de room si configuré
            module_config = self._get_module_config_by_namespace(namespace)
            if module_config and 'room-prefix' in module_config.get('websocket', {}):
                room = f"{module_config['websocket']['room-prefix']}:{room}"

            if room not in self._rooms:
                self._rooms[room] = []
            
            if connection_id not in self._rooms[room]:
                self._rooms[room].append(connection_id)

    def leave_room(self, room: str, websocket: WebSocket, namespace: str = '/'):
        """Retire un client d'une room"""
        connection_id = str(id(websocket))
        
        with self._lock:
            # Applique le préfixe de room si configuré
            module_config = self._get_module_config_by_namespace(namespace)
            if module_config and 'room-prefix' in module_config.get('websocket', {}):
                room = f"{module_config['websocket']['room-prefix']}:{room}"

            if room in self._rooms and connection_id in self._rooms[room]:
                self._rooms[room].remove(connection_id)
                if not self._rooms[room]:  # Supprime la room si vide
                    del self._rooms[room]

    def _get_module_config_by_namespace(self, namespace: str) -> Optional[Dict]:
        """Trouve la configuration du module correspondant à un namespace"""
        with self._lock:
            for module_name, config in self._config._configs.items():
                websocket_config = config.get('websocket', {})
                if 'namespace-prefix' in websocket_config and websocket_config['namespace-prefix'] == namespace:
                    return config
                if 'module-prefix' in websocket_config and websocket_config['module-prefix'] == namespace:
                    return config
                if module_name != 'global' and namespace == f"/{module_name}":
                    return config
            return None

    async def broadcast(self, event: str, data: Any, namespace: str = None):
        """Diffuse un événement à tous les clients connectés"""
        namespaces = [namespace] if namespace else list(self._connections.keys())
        
        for ns in namespaces:
            await self.emit(event, data, ns)

    def get_connected_clients(self, namespace: str = '/') -> List[WebSocket]:
        """Retourne la liste des clients connectés à un namespace"""
        with self._lock:
            return list(self._connections.get(namespace, {}).values())

    def get_rooms(self) -> Dict[str, List[str]]:
        """Retourne la liste des rooms et leurs membres"""
        with self._lock:
            return self._rooms.copy()

    def get_namespaces(self) -> List[str]:
        """Retourne la liste des namespaces actifs"""
        with self._lock:
            return list(self._connections.keys())


import importlib
import inspect
from pathlib import Path
from typing import Any, Callable, Dict, Optional, Union, List
import logging
import asyncio
import threading
from .config_manager import ConfigManager
from .websocket_manager import WebSocketManager
from .websocket_utils import *

logger = logging.getLogger(__name__)

class FeatureManager:
    _instance = None
    _lock = threading.Lock()
    _features_cache: Dict[str, Dict[str, Callable]] = {
        'global': {},
        'modules': {}
    }

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._project_root = None
        self._config = ConfigManager()

    def set_project_root(self, path: Union[str, Path]):
        """Définit la racine du projet"""
        self._project_root = Path(path).resolve()

    async def execute(
        self,
        feature_path: str,
        module: Optional[str] = None,
        *args,
        **kwargs
    ) -> Any:
        """Exécute une fonctionnalité"""
        try:
            feature = self._get_feature(feature_path, module)
            
            # Vérifie si c'est une fonction WebSocket
            if feature_path.startswith('ws.'):
                # Injecte le WebSocketManager si nécessaire
                if 'websocket' not in kwargs:
                    kwargs['websocket'] = WebSocketManager()
                if 'emit' not in kwargs:
                    kwargs['emit'] = emit
                if 'broadcast' not in kwargs:
                    kwargs['broadcast'] = broadcast
            
            if asyncio.iscoroutinefunction(feature):
                return await feature(*args, **kwargs)
            return feature(*args, **kwargs)
            
        except ImportError as e:
            logger.error(f"Feature import error: {e}")
            raise FeatureNotFound(f"Feature '{feature_path}' not found") from e
        except Exception as e:
            logger.error(f"Feature execution error: {e}")
            raise FeatureError(f"Error executing feature '{feature_path}': {str(e)}") from e

    async def execute_many(
        self,
        features: List[Dict[str, Any]],
        fail_fast: bool = False
    ) -> Dict[str, Any]:
        """Exécute plusieurs fonctionnalités en parallèle"""
        tasks = []
        feature_refs = []  # Pour garder la correspondance
        
        for feature_spec in features:
            try:
                feature_path = feature_spec['feature_path']
                feature_module = feature_spec.get('module')
                feature = self._get_feature(
                    feature_path,
                    feature_module
                )
                
                args = feature_spec.get('args', [])
                kwargs = feature_spec.get('kwargs', {})
            
                # Vérifie si c'est une fonction WebSocket
                if feature_path.startswith('ws.'):
                    # Injecte le WebSocketManager si nécessaire
                    if 'websocket' not in kwargs:
                        kwargs['websocket'] = WebSocketManager()
                    if 'emit' not in kwargs:
                        kwargs['emit'] = emit
                    if 'broadcast' not in kwargs:
                        kwargs['broadcast'] = broadcast
                
                if asyncio.iscoroutinefunction(feature):
                    task = feature(*args, **kwargs)
                else:
                    # Convertit les fonctions sync en coroutines
                    task = asyncio.get_event_loop().run_in_executor(
                        None, 
                        lambda: feature(*args, **kwargs)
                    )
                
                tasks.append(task)
                feature_refs.append(feature_spec['feature_path'])
                
            except Exception as e:
                if fail_fast:
                    raise FeatureError(f"Failed to prepare feature {feature_spec['feature_path']}: {str(e)}") from e
                logger.error(f"Feature preparation error: {e}")
                continue
        
        results = {}
        errors = {}
        
        # Exécute toutes les tâches en parallèle
        completed = await asyncio.gather(*tasks, return_exceptions=True)
        
        for feature_path, result in zip(feature_refs, completed):
            if isinstance(result, Exception):
                errors[feature_path] = str(result)
                logger.error(f"Feature {feature_path} failed: {result}")
            else:
                results[feature_path] = result
        
        return {
            'results': results,
            'errors': errors
        }

    async def execute_many_sync(
        self,
        features: List[Dict[str, Any]],
        fail_fast: bool = False
    ) -> Dict[str, Any]:
        """Version synchrone de execute_many"""
        results = {}
        errors = {}
        
        for feature_spec in features:
            try:
                result = await self.execute(
                    feature_spec['feature_path'],
                    feature_spec.get('module'),
                    *feature_spec.get('args', []),
                    **feature_spec.get('kwargs', {})
                )
                results[feature_spec['feature_path']] = result
            except Exception as e:
                if fail_fast:
                    raise
                errors[feature_spec['feature_path']] = str(e)
                logger.error(f"Feature {feature_spec['feature_path']} failed: {e}")
        
        return {
            'results': results,
            'errors': errors
        }

    def _get_feature(self, feature_path: str, module: Optional[str] = None) -> Callable:
        """Récupère une fonctionnalité depuis le cache ou la charge"""
        # Vérifie que le module est valide
        if module and module not in self._config.get_modules_metadata():
            raise FeatureNotFound(f"Module {module} is not valid or not loaded")
            
        context = 'global' if module is None else f'modules.{module}'
        cache_key = f"{context}.{feature_path}"
        
        # Vérifie le cache
        cached_feature = self._features_cache['global' if module is None else 'modules'].get(cache_key)
        if cached_feature:
            return cached_feature
        
        # Charge la fonctionnalité
        feature = self._load_feature(feature_path, module)
        
        # Met en cache
        if module:
            self._features_cache['modules'][cache_key] = feature
        else:
            self._features_cache['global'][cache_key] = feature
            
        return feature

    def _load_feature(self, feature_path: str, module: Optional[str]) -> Callable:
        """Charge dynamiquement une fonctionnalité"""
        parts = feature_path.split('.')
        if len(parts) < 2:
            raise FeatureNotFound(f"Invalid feature path format: {feature_path}")
        
        # Construction du chemin d'import
        if module:
            # Fonctionnalité de module: "auth.createData"
            filename, *func_parts = parts
            import_path = f"modules.{module}.features.{filename}"
            feature_file = self._project_root / "modules" / module / "features" / f"{filename}.py"
        else:
            # Fonctionnalité globale: "utils.helpers.cleanData"
            *path_parts, filename = parts[:-1]
            func_name = parts[-1]
            import_path = "features." + ".".join(path_parts + [filename]) if path_parts else f"features.{filename}"
            feature_file = self._project_root / "features" / "/".join(path_parts) / f"{filename}.py"
        
        # Vérifie l'existence du fichier
        if not feature_file.exists():
            raise FeatureNotFound(f"Feature file not found: {feature_file}")
        
        # Import dynamique
        try:
            feature_module = importlib.import_module(import_path)
            func_name = parts[-1]
            
            if not hasattr(feature_module, func_name):
                raise FeatureNotFound(f"Function '{func_name}' not found in {import_path}")
                
            return getattr(feature_module, func_name)
            
        except ImportError as e:
            logger.error(f"Failed to import feature module {import_path}: {e}")
            raise FeatureNotFound(f"Could not load feature '{feature_path}'") from e

    def list_all_features(self) -> Dict[str, Dict[str, List[str]]]:
        """Liste toutes les fonctionnalités disponibles"""
        features = {'global': {}, 'modules': {}}
        
        # Fonctionnalités globales
        global_features_dir = self._project_root / "features"
        if global_features_dir.exists():
            features['global'] = self._scan_features_dir(global_features_dir, "features")
        
        # Fonctionnalités des modules (seulement ceux valides)
        modules_metadata = self._config.get_modules_metadata()
        modules_dir = self._project_root / "modules"
        if modules_dir.exists():
            for module_name in modules_metadata.keys():
                features_dir = modules_dir / module_name / "features"
                if features_dir.exists():
                    features['modules'][module_name] = self._scan_features_dir(
                        features_dir, 
                        f"modules.{module_name}.features"
                    )
        
        return features

    def list_features(self, module: str = None) -> Dict[str, list]:
        """Liste toutes les fonctionnalités disponibles d'un module"""
        if module is None:
            res = {}
            global_features_dir = self._project_root / "features"
            if global_features_dir.exists():
                res = self._scan_features_dir(global_features_dir, "features")
            return res

        # Vérifie que le module est valide
        if module not in self._config.get_modules_metadata():
            return {}
            
        features_dir = self._project_root / "modules" / module / "features"
        if not features_dir.exists():
            return {}
            
        features = {}
        for feature_file in features_dir.glob('*.py'):
            if feature_file.name.startswith('_'):
                continue
                
            module_name = feature_file.stem
            module_path = f"modules.{module}.features.{module_name}"
            
            try:
                feature_module = importlib.import_module(module_path)
                module_features = [
                    name for name, obj in inspect.getmembers(feature_module)
                    if inspect.isfunction(obj) or inspect.iscoroutinefunction(obj)
                ]
                features[module_name] = module_features
            except ImportError:
                continue
                
        return features

    def _scan_features_dir(self, directory: Path, base_import: str) -> Dict[str, List[str]]:
        """Scan un répertoire de fonctionnalités"""
        features = {}
        
        for feature_file in directory.glob('**/*.py'):
            if feature_file.name.startswith('_'):
                continue
                
            # Construit le chemin d'import
            rel_path = feature_file.relative_to(directory).with_suffix('')
            module_path = f"{base_import}.{'.'.join(rel_path.parts)}"
            
            try:
                feature_module = importlib.import_module(module_path)
                module_features = [
                    name for name, obj in inspect.getmembers(feature_module)
                    if inspect.isfunction(obj) or inspect.iscoroutinefunction(obj)
                ]
                
                if module_features:
                    features[module_path[len(base_import)+1:]] = module_features
            except ImportError:
                continue
                
        return features

class FeatureNotFound(Exception):
    """Exception levée quand une fonctionnalité n'est pas trouvée"""
    pass

class FeatureError(Exception):
    """Exception levée quand une erreur survient lors de l'exécution"""
    pass


import re
from fastapi import Request, HTTPException, UploadFile
from typing import Dict, Any, Optional, Union, List
from pydantic import BaseModel
import json
import traceback
import sys
from collections import defaultdict


class RequestData(BaseModel):
    queries: Dict[str, Any]
    params: Dict[str, Any]
    headers: Dict[str, Any]
    cookies: Dict[str, Any]
    session: Dict[str, Any]
    body: Optional[Union[Dict[str, Any], str]]
    form_data: Optional[Dict[str, Any]]

async def extract_all_request_data(request: Request) -> RequestData:
    try:
        # Extraction des données de base
        query_params = dict(request.query_params)
        path_params = dict(request.path_params)
        headers = dict(request.headers)
        cookies = dict(request.cookies)
        
        # Gestion du corps et form-data
        body = None
        form_data = None
        content_type = headers.get('content-type', '')

        if 'multipart/form-data' in content_type:
            form_data = await parse_form_data(request)
        elif 'application/x-www-form-urlencoded' in content_type:
            form_data = await parse_urlencoded_form(request)
        elif request.method in ['POST', 'PUT', 'PATCH']:
            try:
                body = await request.json()
            except json.JSONDecodeError:
                body = await request.body()
                try:
                    body = body.decode()
                except:
                    body = str(body)

        # Session data
        session = {}
        if hasattr(request.state, 'session'):
            session = request.state.session
        elif 'session' in request.scope:
            session = request.scope['session']

        return RequestData(
            queries=query_params,
            params=path_params,
            headers=headers,
            cookies=cookies,
            session=session,
            body=body,
            form_data=form_data
        )
    except Exception as e:
        stack = traceback.format_exc()
        trace = sys.exc_info()[2]
        print(f"[pyarccore -> request_utils.py] extract_all_request_data | stack:: ", stack)
        raise HTTPException(
            status_code=500,
            detail=f"Failed to extract request data: {str(stack)}"
        )

async def parse_form_data(request) -> Dict[str, Any]:
    form_data = {}
    temp_arrays = defaultdict(list)
    form_items = await request.form()
    
    # Détection automatique du format utilisé
    is_indexed_format = any('[' in key for key in form_items.keys())
    
    for key, value in form_items.items():
        if is_indexed_format:
            # Traitement du format indicé (imgs[0], imgs[1], etc.)
            if match := re.match(r'^(.+?)\[(\d+)\]$', key):
                array_key = match.group(1)
                index = int(match.group(2))
                
                if isinstance(value, list):  # Multiple files in same key
                    temp_arrays[array_key].extend((index, file) for file in value)
                else:
                    temp_arrays[array_key].append((index, value))
            else:
                _process_regular_field(form_data, key, value)
        else:
            # Traitement du format champ multiple (imgs envoyé plusieurs fois)
            _process_regular_field(form_data, key, value)
    
    # Construction des tableaux pour le format indicé
    if is_indexed_format:
        for array_key, items in temp_arrays.items():
            max_index = max(idx for idx, _ in items) if items else 0
            result_array = [None] * (max_index + 1)
            
            for index, value in items:
                if result_array[index] is None:
                    result_array[index] = value
                elif isinstance(result_array[index], list):
                    result_array[index].append(value)
                else:
                    result_array[index] = [result_array[index], value]
            
            form_data[array_key] = [item for item in result_array if item is not None]
    
    return form_data

def _process_regular_field(data: dict, field: str, value: Any):
    """Gère les champs réguliers et les champs multiples"""
    if field in data:  # Champ dupliqué -> convertir en tableau
        existing = data[field]
        if isinstance(existing, list):
            existing.append(_convert_value(value))
        else:
            data[field] = [existing, _convert_value(value)]
    else:
        data[field] = _convert_value(value)

def _convert_value(value: Any) -> Any:
    """Conversion intelligente des valeurs"""
    if isinstance(value, UploadFile):
        return value
    if isinstance(value, str):
        value = value.strip()
        if value.isdigit():
            return int(value)
        try:
            return float(value)
        except ValueError:
            if value.lower() in ('true', 'false'):
                return value.lower() == 'true'
    return value

async def parse_urlencoded_form(request: Request) -> Dict[str, Any]:
    body = await request.body()
    form_data = {}
    for item in body.decode().split('&'):
        key, value = item.split('=', 1)
        _process_form_field(form_data, key, value)
    return form_data

def _process_form_field(data: dict, key: str, value: Any):
    # Gestion des tableaux (champs comme 'imgs[]')
    if key.endswith('[]'):
        clean_key = key[:-2]
        if clean_key not in data:
            data[clean_key] = []
        data[clean_key].append(value)
        return

    # Gestion des structures imbriquées (other_info[data1])
    if '[' in key and ']' in key:
        parts = re.split(r'\[|\]', key)
        parts = [p for p in parts if p]  # Supprime les chaînes vides
        
        current = data
        for part in parts[:-1]:
            if part not in current:
                current[part] = {}
            current = current[part]
        
        last_part = parts[-1]
        current[last_part] = _convert_value(value)
    else:
        data[key] = _convert_value(value)

def _convert_value(value: Any) -> Any:
    if isinstance(value, UploadFile):
        return value
    
    if isinstance(value, str):
        value = value.strip()
        if value.isdigit():
            return int(value)
        try:
            return float(value)
        except ValueError:
            if value.lower() in ('true', 'false'):
                return value.lower() == 'true'
    return value


def get_specific_data(
    request_data: RequestData,
    data_type: str,
    key: Optional[str] = None,
    default: Any = None
) -> Any:
    """
    Récupère une donnée spécifique brute sans traitement
    """
    data_map = {
        'queries': request_data.queries,
        'params': request_data.params,
        'headers': request_data.headers,
        'cookies': request_data.cookies,
        'session': request_data.session,
        'body': request_data.body,
        'form_data': request_data.form_data or {}
    }
    
    if data_type not in data_map:
        raise HTTPException(
            status_code=400,
            detail=f"Invalid data type. Must be one of: {', '.join(data_map.keys())}"
        )
    
    data = data_map[data_type]
    
    if key is not None:
        if data_type == 'form_data' and key in data:
            return data[key].value  # Retourne la valeur brute
        return data.get(key, default)
    return data


from packaging import version
from typing import Union

def check_version(installed_version: str, version_spec: str) -> bool:
    """
    Vérifie rigoureusement qu'une version satisfait une spécification.
    
    Args:
        installed_version: Version installée (ex: '1.2.3')
        version_spec: Spécification de version (ex: '>=1.0,<2.0 || 2.5.0')
    
    Returns:
        bool: True si la version est compatible
    """
    try:
        installed_ver = version.parse(installed_version)
        version_spec = version_spec.strip()
        
        if not version_spec:  # Aucune spécification = tout est accepté
            return True
            
        # Gestion des OU logiques (||)
        if '||' in version_spec:
            return any(
                _process_spec(installed_ver, part.strip())
                for part in version_spec.split('||')
            )
        
        return _process_spec(installed_ver, version_spec)
    except Exception as e:
        raise ValueError(f"Version check failed for '{installed_version}' with spec '{version_spec}': {str(e)}")

def _process_spec(ver: version.Version, spec: str) -> bool:
    """Traite une spécification individuelle (peut contenir des ET)"""
    # Gestion des ET logiques (&&)
    if '&&' in spec:
        return all(
            _compare(ver, part.strip())
            for part in spec.split('&&')
        )
    
    # Gestion des intervalles avec virgule
    if ',' in spec:
        parts = [p.strip() for p in spec.split(',')]
        if len(parts) == 2:
            return (_compare(ver, f'>={parts[0]}') and 
                    _compare(ver, f'<={parts[1]}'))
    
    return _compare(ver, spec)

def _compare(ver: version.Version, spec: str) -> bool:
    """Compare une version avec une spécification individuelle"""
    spec = spec.strip()
    
    # Détection de l'opérateur
    op = ''
    if spec.startswith(('>=', '<=', '==', '!=')):
        op = spec[:2]
        spec_ver = version.parse(spec[2:].strip())
    elif spec.startswith(('>', '<', '~', '^')):
        op = spec[0]
        spec_ver = version.parse(spec[1:].strip())
    else:
        # Version exacte si pas d'opérateur
        return ver == version.parse(spec)
    
    # Comparaisons
    if op == '>':
        return ver > spec_ver
    elif op == '>=':
        return ver >= spec_ver
    elif op == '<':
        return ver < spec_ver
    elif op == '<=':
        return ver <= spec_ver
    elif op == '==':
        return ver == spec_ver
    elif op == '!=':
        return ver != spec_ver
    elif op == '~':  # Compatible release (pip style)
        return ver >= spec_ver and ver < version.parse(f"{spec_ver.major}.{spec_ver.minor + 1}.0")
    elif op == '^':  # Caret requirement (npm style)
        if spec_ver.major > 0:
            return ver >= spec_ver and ver < version.parse(f"{spec_ver.major + 1}.0.0")
        elif spec_ver.minor > 0:
            return ver >= spec_ver and ver < version.parse(f"0.{spec_ver.minor + 1}.0")
        else:
            return ver >= spec_ver and ver < version.parse(f"0.0.{spec_ver.micro + 1}")
    
    return False


from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from pathlib import Path
import os
import threading
from typing import Dict, List, Optional
from fastapi import Request
from fastapi.responses import HTMLResponse
import logging
from .config_manager import ConfigManager

logger = logging.getLogger(__name__)

class PageManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._project_root = None
        self._config = ConfigManager()
        self._templates = None
        self._static_mounts = []
        self._lock = threading.RLock()

    def set_project_root(self, path: Path):
        """Définit la racine du projet"""
        with self._lock:
            self._project_root = path.resolve()
            self._init_templates()
            self._init_static_mounts()

    def _init_templates(self):
        """Initialise les templates Jinja2"""
        template_dirs = []
        
        # Global pages
        global_pages = self._project_root / 'pages'
        if global_pages.exists():
            template_dirs.append(str(global_pages))
        
        # Modules pages
        modules_metadata = self._config.get_modules_metadata()
        modules_dir = self._project_root / 'modules'
        if modules_dir.exists():
            for module_name in modules_metadata.keys():
                module_pages = modules_dir / module_name / 'pages'
                if module_pages.exists():
                    template_dirs.append(str(module_pages))
        
        if template_dirs:
            self._templates = Jinja2Templates(directory=template_dirs)

    @staticmethod
    def _get_base_url(config_data: dict, initial_path: str = "") -> str:
        """Extrait le base-url de la configuration"""
        try:
            routing = config_data.get('routing', {})
            base_url = routing.get('base-url', '').strip()
            if base_url and not base_url.startswith('/'):
                base_url = f'/{base_url}'
            if not(
                type(base_url) == str and
                len(base_url) > 0
            ):
                base_url = initial_path
            base_url = "/".join(base_url.split('//'))
            if (
                not(
                    type(base_url) == str and
                    len(base_url) > 0
                ) or base_url == '/'
            ):
                base_url = ""
            logger.debug(f"Extracted base-url: {base_url}")  # Ajout pour debug
            return base_url
        except Exception as e:
            logger.error(f"Error extracting base-url: {e}")
            return ''
    def _init_static_mounts(self):
        """Initialise les chemins pour les fichiers statiques"""
        self._static_mounts = []
        
        # Modules public - seulement ceux valides
        modules_metadata = self._config.get_modules_metadata()
        modules_dir = self._project_root / 'modules'
        
        if modules_dir.exists():
            for module_name in modules_metadata.keys():
                module_config = self._config._configs.get(module_name, {})
                module_base_url = self._get_base_url(module_config, initial_path=f"/{module_name}")
                module_public = modules_dir / module_name / 'public'
                if module_public.exists():
                    finalModuleUrl = "/".join(f'/assets/{module_base_url}'.split('//'))
                    self._static_mounts.append(
                        (finalModuleUrl, module_public)
                    )
                    logger.info(f"Mounted static files for module {module_name} at {finalModuleUrl}")
        
        # Global public
        global_public = self._project_root / 'public'
        if global_public.exists():
            self._static_mounts.append(('/assets', global_public))
            logger.info(f"Mounted static files at /assets")
    def mount_static_files(self, app):
        """Monte les fichiers statiques sur l'application FastAPI"""
        with self._lock:
            for route, path in self._static_mounts:
                app.mount(route, StaticFiles(directory=path), name=f"static_{path.name}")

    def render_template(self, request: Request, template_name: str, context: str = 'global', **kwargs) -> HTMLResponse:
        """
        Rend un template avec le contexte de la requête
        
        Args:
            request: Objet Request de FastAPI
            template_name: Nom du template ('index.html', 'auth/login.html')
            context: 'global' ou nom du module
            **kwargs: Variables supplémentaires pour le template
            
        Returns:
            HTMLResponse
        """
        with self._lock:
            if self._templates is None:
                raise ValueError("Templates not initialized")
                
            # Ajoute les URLs des assets au contexte
            kwargs.update({
                'request': request,
                'static_url': self.get_static_url(context)
            })
            
            try:
                # Essaye d'abord avec le contexte spécifié
                template_path = f"{context}/{template_name}" if context != 'global' else template_name
                return self._templates.TemplateResponse(template_path, kwargs)
            except Exception as e:
                # Fallback global si le template n'est pas trouvé dans le module
                if context != 'global':
                    try:
                        return self._templates.TemplateResponse(template_name, kwargs)
                    except Exception:
                        pass
                logger.error(f"Error rendering template {template_name}: {e}")
                raise

    def get_static_url(self, context: str = 'global') -> str:
        """
        Retourne l'URL de base pour les assets statiques
        
        Args:
            context: 'global' ou nom du module
            
        Returns:
            URL de base pour les assets
        """
        if context == 'global':
            return '/assets'
        return f'/assets/{context}'

    def list_templates(self, context: str = None) -> Dict[str, List[str]]:
        """
        Liste tous les templates disponibles
        
        Args:
            context: 'global', nom de module ou None pour tous
            
        Returns:
            Dictionnaire des templates disponibles
        """
        with self._lock:
            if self._templates is None:
                return {}
                
            result = {}
            template_dirs = self._templates.env.loader.searchpath
            
            for template_dir in template_dirs:
                ctx = 'global'
                if 'modules' in template_dir:
                    ctx = Path(template_dir).parent.name
                
                if context is not None and ctx != context:
                    continue
                    
                templates = []
                for root, _, files in os.walk(template_dir):
                    for file in files:
                        if file.endswith('.html'):
                            rel_path = os.path.relpath(os.path.join(root, file), template_dir)
                            templates.append(rel_path.replace('\\', '/'))
                
                if ctx not in result:
                    result[ctx] = []
                result[ctx].extend(templates)
            
            return result


import asyncio
import inspect
import logging
import threading
from pathlib import Path
from typing import Any, Dict, List, Optional, Union, Callable
from fastapi import WebSocket, WebSocketDisconnect
from fastapi.websockets import WebSocketState
from .config_manager import ConfigManager
import importlib
import json
from collections import defaultdict
from uuid import uuid4

logger = logging.getLogger(__name__)

class ConnectionManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self.active_connections: Dict[str, WebSocket] = {}
        self.rooms: Dict[str, Dict[str, WebSocket]] = defaultdict(dict)
        self.namespaces: Dict[str, Dict[str, WebSocket]] = defaultdict(dict)
        self.event_handlers: Dict[str, Dict[str, Callable]] = defaultdict(dict)
        self._config = ConfigManager()
        self._prefixes = {
            'event': {},
            'room': {},
            'namespace': {}
        }

    async def connect(self, websocket: WebSocket, client_id: str, namespace: str = "/"):
        await websocket.accept()
        self.active_connections[client_id] = websocket
        self.namespaces[namespace][client_id] = websocket
        return client_id

    def disconnect(self, client_id: str, namespace: str = "/"):
        if client_id in self.active_connections:
            del self.active_connections[client_id]
        if namespace in self.namespaces and client_id in self.namespaces[namespace]:
            del self.namespaces[namespace][client_id]
        
        # Remove from all rooms
        for room in list(self.rooms.keys()):
            if client_id in self.rooms[room]:
                del self.rooms[room][client_id]

    async def send_personal_message(self, message: str, client_id: str):
        if client_id in self.active_connections:
            websocket = self.active_connections[client_id]
            if websocket.client_state == WebSocketState.CONNECTED:
                await websocket.send_text(message)

    async def broadcast(self, message: str, namespace: str = "/"):
        for client_id, websocket in self.namespaces[namespace].items():
            if websocket.client_state == WebSocketState.CONNECTED:
                await websocket.send_text(message)

    async def send_to_room(self, room: str, message: str, namespace: str = "/"):
        for client_id, websocket in self.rooms[room].items():
            if (websocket.client_state == WebSocketState.CONNECTED and 
                client_id in self.namespaces[namespace]):
                await websocket.send_text(message)

    async def add_to_room(self, room: str, client_id: str):
        if client_id in self.active_connections:
            self.rooms[room][client_id] = self.active_connections[client_id]

    def remove_from_room(self, room: str, client_id: str):
        if room in self.rooms and client_id in self.rooms[room]:
            del self.rooms[room][client_id]

    def register_event(self, event: str, handler: Callable, namespace: str = "/", module: str = None):
        prefixed_event = self._apply_prefix(event, 'event', module)
        self.event_handlers[namespace][prefixed_event] = handler

    async def handle_event(self, event: str, data: Any, client_id: str, namespace: str = "/", module: str = None):
        prefixed_event = self._apply_prefix(event, 'event', module)
        if namespace in self.event_handlers and prefixed_event in self.event_handlers[namespace]:
            handler = self.event_handlers[namespace][prefixed_event]
            if inspect.iscoroutinefunction(handler):
                return await handler(data, client_id)
            return handler(data, client_id)
        return None

    def _apply_prefix(self, name: str, prefix_type: str, module: str = None):
        if module and module in self._prefixes[prefix_type]:
            return f"{self._prefixes[prefix_type][module]}-{name}"
        return name

    def load_prefixes(self):
        modules_metadata = self._config.get_modules_metadata()
        for module_name, config in self._config._configs.items():
            if module_name == 'global' or module_name not in modules_metadata:
                continue
                
            websocket_config = config.get('websocket', {})
            if 'event-prefix' in websocket_config:
                self._prefixes['event'][module_name] = websocket_config['event-prefix']
            if 'room-prefix' in websocket_config:
                self._prefixes['room'][module_name] = websocket_config['room-prefix']
            if 'namespace-prefix' in websocket_config:
                self._prefixes['namespace'][module_name] = websocket_config['namespace-prefix']

class WebSocketManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._project_root = None
        self._config = ConfigManager()
        self._connection_manager = ConnectionManager()
        self._initialized = False
        self._handlers = {}
        self._lock = threading.RLock()

    def set_project_root(self, path: Union[str, Path]):
        """Définit la racine du projet"""
        self._project_root = Path(path).resolve()

    def initialize(self):
        """Initialise le WebSocketManager"""
        with self._lock:
            if self._initialized:
                return
                
            self._connection_manager.load_prefixes()
            self._load_handlers()
            self._initialized = True

    def _load_handlers(self):
        """Charge les handlers WebSocket depuis les fichiers ws/"""
        # Global handlers
        global_ws = self._project_root / 'ws'
        if global_ws.exists():
            self._load_ws_handlers_from_dir(global_ws, 'global')

        # Module handlers
        modules_metadata = self._config.get_modules_metadata()
        modules_dir = self._project_root / 'modules'
        if modules_dir.exists():
            for module_name in modules_metadata.keys():
                module_ws = modules_dir / module_name / 'ws'
                if module_ws.exists():
                    self._load_ws_handlers_from_dir(module_ws, module_name)

    def _load_ws_handlers_from_dir(self, directory: Path, context: str):
        """Charge les handlers depuis un répertoire ws/"""
        for handler_file in directory.glob('**/*.py'):
            if handler_file.name.startswith('_'):
                continue

            try:
                module_path = '.'.join([
                    'modules', context, 'ws', 
                    *handler_file.relative_to(directory.parent).with_suffix('').parts
                ]) if context != 'global' else '.'.join([
                    'ws', 
                    *handler_file.relative_to(self._project_root).with_suffix('').parts
                ])

                module = importlib.import_module(module_path)
                
                # Enregistre tous les handlers du module
                for name, obj in inspect.getmembers(module):
                    if name.startswith('handle_') and callable(obj):
                        event_name = name[7:]  # Enlève 'handle_'
                        namespace = getattr(obj, '__namespace__', '/')
                        self._connection_manager.register_event(
                            event_name, obj, namespace, context
                        )
                        logger.info(f"Registered WS handler: {namespace}/{event_name} from {context}")

            except Exception as e:
                logger.error(f"Failed to load WS handler from {handler_file}: {e}")

    async def handle_connection(self, websocket: WebSocket, namespace: str = "/"):
        """Gère une connexion WebSocket"""
        client_id = str(uuid4())
        await self._connection_manager.connect(websocket, client_id, namespace)
        
        try:
            while True:
                data = await websocket.receive_text()
                try:
                    message = json.loads(data)
                    if isinstance(message, dict) and 'event' in message:
                        await self._handle_ws_message(message, client_id, namespace)
                except json.JSONDecodeError:
                    logger.warning(f"Invalid JSON received from {client_id}")
        except WebSocketDisconnect:
            self._connection_manager.disconnect(client_id, namespace)
            logger.info(f"Client disconnected: {client_id}")

    async def _handle_ws_message(self, message: dict, client_id: str, namespace: str = "/"):
        """Traite un message WebSocket"""
        event = message['event']
        data = message.get('data')
        module = message.get('module')
        
        result = await self._connection_manager.handle_event(
            event, data, client_id, namespace, module
        )
        
        if result is not None:
            await self.send_to_client(client_id, {
                'event': f"{event}_response",
                'data': result
            }, namespace)

    async def send_to_client(self, client_id: str, message: dict, namespace: str = "/"):
        """Envoie un message à un client spécifique"""
        await self._connection_manager.send_personal_message(
            json.dumps(message), client_id
        )

    async def broadcast(self, message: dict, namespace: str = "/"):
        """Diffuse un message à tous les clients d'un namespace"""
        await self._connection_manager.broadcast(
            json.dumps(message), namespace
        )

    async def send_to_room(self, room: str, message: dict, namespace: str = "/"):
        """Envoie un message à tous les clients d'une room"""
        await self._connection_manager.send_to_room(
            room, json.dumps(message), namespace
        )

    async def add_to_room(self, room: str, client_id: str):
        """Ajoute un client à une room"""
        self._connection_manager.add_to_room(room, client_id)

    def remove_from_room(self, room: str, client_id: str):
        """Retire un client d'une room"""
        self._connection_manager.remove_from_room(room, client_id)

    def get_client_ids(self, namespace: str = "/") -> List[str]:
        """Retourne les IDs des clients connectés à un namespace"""
        return list(self._connection_manager.namespaces[namespace].keys())

    def get_room_clients(self, room: str) -> List[str]:
        """Retourne les IDs des clients dans une room"""
        return list(self._connection_manager.rooms[room].keys())

def websocket_handler(namespace: str = "/"):
    """Décorateur pour enregistrer un handler WebSocket"""
    def decorator(func):
        func.__namespace__ = namespace
        return func
    return decorator


from typing import Any, Union, Dict, List, Optional
from fastapi import FastAPI, APIRouter, Request, WebSocket
from fastapi.responses import HTMLResponse
from pathlib import Path
from .internationalisation_manager import InternationalisationManager
from .config_manager import ConfigManager
from .router import ArcCmsRouter
from .websocket_manager import WebSocketManager
from .page_manager import PageManager
from .feature_manager import FeatureManager
from .request_utils import RequestData, extract_all_request_data, get_specific_data
from .http_middleware_manager import HttpMiddlewareManager
from .ws_middleware_manager import WsMiddlewareManager
import logging
import asyncio

logger = logging.getLogger(__name__)

_intl = InternationalisationManager()
_config = ConfigManager()
_router = ArcCmsRouter()
_feature = FeatureManager()
_page = PageManager()
_websocket = WebSocketManager()
_http_middleware = HttpMiddlewareManager()
_ws_middleware = WsMiddlewareManager()

def init_app(app_root: Union[Path, str] = None):
    """Initialise toutes les ressources"""
    if app_root is None:
        app_root = Path(__file__).parent.parent
    if isinstance(app_root, str):
        app_root = Path(app_root)
    
    _intl.set_project_root(app_root.resolve())
    _config.set_project_root(app_root.resolve())
    _feature.set_project_root(app_root)
    _websocket.set_project_root(app_root.resolve())
    _intl.load_all()
    _config.load_all()
    _page.set_project_root(app_root.resolve())
    _http_middleware.set_project_root(app_root.resolve())
    _ws_middleware.set_project_root(app_root.resolve())
    
def execute_feature(
    feature_path: str,
    module: Optional[str] = None,
    *args,
    **kwargs
) -> Any:
    """Exécute une fonctionnalité"""
    return _feature.execute(feature_path, module, *args, **kwargs)

async def execute_features(
    features: List[Dict[str, Any]],
    fail_fast: bool = False
) -> Dict[str, Any]:
    """Exécute plusieurs fonctionnalités en parallèle"""
    return await _feature.execute_many(features, fail_fast)

def execute_features_sync(
    features: List[Dict[str, Any]],
    fail_fast: bool = False
) -> Dict[str, Any]:
    """Version synchrone de execute_features"""
    return asyncio.run(_feature.execute_many_sync(features, fail_fast))

def list_features(module: str = None) -> Dict[str, list]:
    """Liste toutes les fonctionnalités disponibles d'un module"""
    return _feature.list_features(module)

def list_all_features() -> Dict[str, Dict[str, List[str]]]:
    """Liste toutes les fonctionnalités disponibles"""
    return _feature.list_all_features()

def t(key: str, locale: str = 'fr', module: str = "global", **kwargs) -> str:
    """Récupère une traduction"""
    value = _intl.get(module, key, locale) or key
    return value.format(**kwargs) if kwargs else value

def cfg(key: str, default: Any = None, module: str = "global") -> Any:
    """Récupère une configuration"""
    return _config.get(module, key, default)

def register_routes(router: APIRouter, base_path: Union[Path, str], is_global: bool = True):
    """Enregistre les routes"""
    if isinstance(base_path, str):
        base_path = Path(base_path)
    _router.register_routes(router, base_path.resolve(), is_global=is_global)

def get_module_id_map() -> Dict[str, str]:
    """Retourne un mapping des IDs vers les noms de dossiers"""
    return _config.get_module_id_map()

async def all_requests(request: Request) -> RequestData:
    return await extract_all_request_data(request = request)

def specific_request(
    request_data: RequestData,
    data_type: str,
    key: Optional[str] = None,
    default: Any = None
) -> Any:
    return get_specific_data(
        request_data = request_data,
        data_type = data_type,
        key = key,
        default = default,
    )

def get_project_root() -> Path:
    """Retourne la racine du projet"""
    return _config.get_project_root()

def get_modules_metadata() -> Dict[str, Dict]:
    """Retourne les métadonnées de tous les modules"""
    return _config.get_modules_metadata()

def get_files_from_directory(
    directory_name: str, 
    module: str = None, 
    file_filter: str = None
) -> Dict[str, List[Path]]:
    """
    Récupère les fichiers d'un répertoire spécifique
    
    Args:
        directory_name: Nom du répertoire ('public', 'pages', etc.)
        module: Nom du module (None pour global)
        file_filter: Regex pour filtrer les fichiers
    """
    return _config.get_files_from_directory(directory_name, module, file_filter)
def get_all_files_from_directory(
    directory_name: str,
    file_filter: str = None,
    include_global: bool = True
) -> Dict[str, List[Path]]:
    """
    Récupère les fichiers d'un répertoire spécifique dans tous les modules
    
    Args:
        directory_name: Nom du répertoire ('public', 'pages', etc.)
        file_filter: Regex pour filtrer les fichiers
        include_global: Inclure les fichiers globaux si True
        
    Returns:
        Dictionnaire avec:
            - clé: nom du module ou 'global'
            - valeur: liste des fichiers trouvés
    """
    return _config.get_all_files_from_directory(
        directory_name=directory_name,
        file_filter=file_filter,
        include_global=include_global,
    )

def get_config_params(
    param_path: str, 
    module: str = None, 
    key_filter: str = None
) -> Dict[str, Any]:
    """
    Récupère des paramètres spécifiques de configuration
    
    Args:
        param_path: Chemin du paramètre ('metadatas.authors')
        module: Nom du module (None pour global)
        key_filter: Regex pour filtrer les clés
    """
    return _config.get_config_params(param_path, module, key_filter)
def get_all_config_params(
    param_path: str,
    key_filter: str = None,
    include_global: bool = True
) -> Dict[str, Dict[str, Any]]:
    """
    Récupère des paramètres spécifiques de configuration pour tous les modules
    
    Args:
        param_path: Chemin du paramètre ('metadatas.authors')
        key_filter: Regex pour filtrer les clés
        include_global: Inclure la configuration globale si True
        
    Returns:
        Dictionnaire avec:
            - clé: nom du module ou 'global'
            - valeur: paramètres trouvés pour ce module
    """
    return _config.get_all_config_params(
        param_path=param_path,
        key_filter=key_filter,
        include_global=include_global,
    )

def get_page(page_path: str, context: str = 'global', **kwargs) -> str:
    """Récupère une page rendue"""
    return _page.get_page(page_path, context, **kwargs)

def list_pages(context: str = None, pattern: str = None) -> Dict[str, List[str]]:
    """Liste toutes les pages disponibles"""
    return _page.list_pages(context, pattern)

def get_layout(layout_name: str, context: str = 'global') -> str:
    """Récupère un layout"""
    return _page.get_layout(layout_name, context)

def list_layouts(context: str = None) -> Dict[str, List[str]]:
    """Liste tous les layouts disponibles"""
    return _page.list_layouts(context)

def mount_static_files(app):
    """Monte les fichiers statiques sur l'application FastAPI"""
    _page.mount_static_files(app)

def render_template(request: Request, template_name: str, context: str = 'global', **kwargs) -> HTMLResponse:
    """
    Rend un template avec le contexte de la requête
    
    Args:
        request: Objet Request de FastAPI
        template_name: Nom du template ('index.html', 'auth/login.html')
        context: 'global' ou nom du module
        **kwargs: Variables supplémentaires pour le template
        
    Returns:
        HTMLResponse
    """
    return _page.render_template(request, template_name, context, **kwargs)

def get_static_url(context: str = 'global') -> str:
    """
    Retourne l'URL de base pour les assets statiques
    
    Args:
        context: 'global' ou nom du module
        
    Returns:
        URL de base pour les assets
    """
    return _page.get_static_url(context)

def list_templates(context: str = None) -> Dict[str, List[str]]:
    """
    Liste tous les templates disponibles
    
    Args:
        context: 'global', nom de module ou None pour tous
        
    Returns:
        Dictionnaire des templates disponibles
    """
    return _page.list_templates(context)

async def initialize_websocket():
    """Initialise le serveur WebSocket"""
    await _websocket.initialize()

def get_websocket_manager() -> WebSocketManager:
    """Retourne le WebSocketManager"""
    return _websocket
from .websocket_utils import *

async def initialize_middlewares():
    """Initialise les middlewares"""
    await _http_middleware.load_all()
    await _ws_middleware.load_all()

def initializer(
    app_root: str,
    title: str="ARC-CMS",
    version: str="0.0.1",
    host: str='0.0.0.0',
    port: int=5464,
):
    from fastapi import FastAPI, APIRouter, Request, WebSocket
    from fastapi.responses import JSONResponse
    from fastapi.middleware.httpsredirect import HTTPSRedirectMiddleware
    from fastapi.middleware.trustedhost import TrustedHostMiddleware
    from fastapi.middleware.gzip import GZipMiddleware
    from pathlib import Path
    import uvicorn
    import os
    import logging
    from typing import Union

    # Configuration du logging
    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        handlers=[
            logging.StreamHandler(),
            logging.FileHandler('app.log')
        ]
    )
    logger = logging.getLogger(__name__)
    logging.getLogger('pyarccore').setLevel(logging.DEBUG)

    # Configuration des workers
    WORKERS = int(os.getenv('WORKERS', 1))
    MAX_THREADS = int(os.getenv('MAX_THREADS', 1000))

    app = FastAPI(
        title=title,
        version=version,
        docs_url=None,
        redoc_url=None
    )

    app.add_middleware(GZipMiddleware)
    app.add_middleware(TrustedHostMiddleware, allowed_hosts=["*"])

    # Route racine
    @app.get("/")
    async def root():
        return {"message": title, "status": "running"}

    # Route de configuration
    @app.get("/config")
    async def get_config():
        return {
            "app_name": title,
            "version": version,
            "status": "active"
        }

    @app.middleware("http")
    async def http_middleware(request: Request, call_next):
        # Détermine le contexte du module
        context = 'global'
        if request.url.path.startswith('/modules/'):
            module_name = request.url.path.split('/')[2]
            if module_name in _config.get_modules_metadata():
                context = module_name
        
        return await _http_middleware.apply_middlewares(request, call_next, context)

    # Route WebSocket
    @app.websocket("/ws")
    async def websocket_endpoint(websocket: WebSocket):
        await _ws_middleware.apply_middlewares(
            websocket,
            lambda ws: _websocket.handle_connection(ws),
            'global'
        )

    # Namespace WebSocket
    @app.websocket("/ws/{namespace}")
    async def websocket_namespace_endpoint(websocket: WebSocket, namespace: str):
        context = namespace.lstrip('/')
        if context in _config.get_modules_metadata():
            await _ws_middleware.apply_middlewares(
                websocket,
                lambda ws: _websocket.handle_connection(ws, f"/{namespace}"),
                context
            )
        else:
            await websocket.close(code=1008)

    @app.on_event("startup")
    async def startup():
        init_app(app_root)
        await load_all_routes()
        mount_static_files(app)
        
        # Initialise WebSocket si configuré
        try:
            protocols = _config.get_protocols()
                
            if 'websocket' in protocols:
                await initialize_websocket()
                logger.info("WebSocket server initialized")
        except Exception as e:
            logger.warning(f"Could not parse protocols configuration: {e}")

        await initialize_middlewares()

    async def load_all_routes():
        try:
            # Routes globales
            global_router = APIRouter()
            
            global_path = app_root
            if global_path.exists():
                register_routes(global_router, global_path, is_global=True)
                app.include_router(global_router)
                logger.info("Global routes loaded successfully")
            else:
                logger.warning(f"Views directory not found: {app_root}")

        except Exception as e:
            logger.error(f"Failed to load routes: {e}")
            raise

    return (
        app_root,
        title,
        version,
        host,
        port,
        WORKERS,
        MAX_THREADS,
        app,
    )


import importlib
import inspect
from pathlib import Path
import threading
from typing import Callable, Dict, List, Optional, Union
from fastapi import Request, Response
import logging
from .config_manager import ConfigManager

logger = logging.getLogger(__name__)

class HttpMiddlewareManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._project_root = None
        self._config = ConfigManager()
        self._middlewares: Dict[str, Dict[str, List[Callable]]] = {
            'global': {'before': [], 'after': [], 'all': []},
            'modules': {}
        }
        self._lock = threading.RLock()

    def set_project_root(self, path: Path):
        with self._lock:
            self._project_root = path.resolve()

    async def load_all(self):
        """Charge tous les middlewares HTTP"""
        with self._lock:
            # Global middlewares
            global_mw_dir = self._project_root / 'middlewares'
            if global_mw_dir.exists():
                self._load_middlewares('global', global_mw_dir)

            # Module middlewares
            modules_metadata = self._config.get_modules_metadata()
            modules_dir = self._project_root / 'modules'
            if modules_dir.exists():
                for module_name in modules_metadata.keys():
                    module_mw_dir = modules_dir / module_name / 'middlewares'
                    if module_mw_dir.exists():
                        self._load_middlewares(module_name, module_mw_dir)

    def _load_middlewares(self, context: str, mw_dir: Path):
        """Charge les middlewares depuis un répertoire"""
        for mw_file in mw_dir.glob('*.py'):
            if mw_file.name.startswith(('_', '.')):
                continue

            try:
                module_path = '.'.join(mw_file.relative_to(mw_dir.parent).with_suffix('').parts)
                module = importlib.import_module(module_path)

                for name, obj in inspect.getmembers(module):
                    if name.startswith(('before_', 'after_', 'all_')):
                        phase = name.split('_')[0]
                        self._register_middleware(context, phase, obj)

            except Exception as e:
                logger.error(f"Failed to load middleware {mw_file}: {e}")

    def _register_middleware(self, context: str, phase: str, middleware: Callable):
        """Enregistre un middleware"""
        with self._lock:
            if context not in self._middlewares['modules']:
                self._middlewares['modules'][context] = {'before': [], 'after': [], 'all': []}
            
            self._middlewares['modules'][context][phase].append(middleware)
            logger.info(f"Registered HTTP {phase} middleware for {context}")

    async def apply_middlewares(self, request: Request, call_next, context: str = 'global') -> Response:
        """Applique la chaîne de middlewares"""
        # Récupère les middlewares dans l'ordre
        with self._lock:
            middlewares = {
                'before': self._middlewares['global']['before'] + 
                         self._middlewares['modules'].get(context, {}).get('before', []),
                'all': self._middlewares['global']['all'] + 
                       self._middlewares['modules'].get(context, {}).get('all', []),
                'after': self._middlewares['global']['after'] + 
                        self._middlewares['modules'].get(context, {}).get('after', [])
            }

        # Exécute les middlewares 'before' et 'all' (avant)
        for phase in ['before', 'all']:
            for middleware in middlewares[phase]:
                request = await self._execute_middleware(middleware, request, None)

        # Exécute la route principale
        response = await call_next(request)

        # Exécute les middlewares 'after' et 'all' (après)
        for phase in ['after', 'all']:
            for middleware in middlewares[phase]:
                response = await self._execute_middleware(middleware, request, response)

        return response

    async def _execute_middleware(self, middleware: Callable, request: Request, response: Optional[Response]):
        """Exécute un middleware individuel"""
        try:
            if inspect.iscoroutinefunction(middleware):
                if response is None:
                    return await middleware(request)
                else:
                    return await middleware(request, response)
            else:
                if response is None:
                    return middleware(request)
                else:
                    return middleware(request, response)
        except Exception as e:
            logger.error(f"Middleware execution failed: {e}")
            raise


import json
import threading
from pathlib import Path
from functools import lru_cache
from typing import Dict, Optional, Any
import logging
from .config_manager import ConfigManager

logger = logging.getLogger(__name__)

class InternationalisationManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._init()
        return cls._instance

    def _init(self):
        self._translations = {}
        self._lock = threading.RLock()
        self._project_root = None
        self._config = ConfigManager()

    def set_project_root(self, path: Path):
        """Définit la racine du projet"""
        with self._lock:
            self._project_root = path.resolve()

    def load_all(self):
        """Charge toutes les traductions"""
        if self._project_root is None:
            logger.error("Project root not set, call set_project_root() first")
            return

        # Global
        global_locale = self._project_root / 'locales'
        if global_locale.exists():
            self._load_translations('global', global_locale)

        # Modules - seulement ceux avec une structure valide
        modules_metadata = self._config.get_modules_metadata()
        modules_dir = self._project_root / 'modules'
        if modules_dir.exists():
            for module_name in modules_metadata.keys():
                module_path = modules_dir / module_name
                locale_dir = module_path / 'locales'
                if locale_dir.exists():
                    self._load_translations(module_name, locale_dir)

    def _load_translations(self, module_name: str, locale_dir: Path):
        """Charge les traductions pour un module"""
        locale_dir = locale_dir.resolve()
        translations = {}
        
        for locale_file in locale_dir.glob('*.json'):
            try:
                with open(locale_file, 'r', encoding='utf-8') as f:
                    content = f.read().strip()
                    if not content:
                        logger.warning(f"Empty translation file: {locale_file}")
                        continue
                    translations[locale_file.stem] = json.loads(content)
            except json.JSONDecodeError as e:
                logger.error(f"Invalid JSON in {locale_file}: {e}")
                continue
            except Exception as e:
                logger.error(f"Error loading {locale_file}: {e}")
                continue
        
        if translations:
            with self._lock:
                self._translations[module_name] = translations

    @lru_cache(maxsize=2048)
    def get(self, module: str, key: str, locale: str = 'fr') -> Optional[str]:
        """Récupère une traduction avec cache"""
        with self._lock:
            keys = key.split('.')
            try:
                value = self._translations.get(module, {}).get(locale, {})
                for k in keys:
                    value = value[k]
                return value
            except (KeyError, TypeError):
                return None


from setuptools import setup, find_packages

setup(
    name="pyarccore",
    version="0.0.1211149",
    author="INICODE",
    author_email="contact.inicode@gmail.com",
    description="Paquetage Python pour gérer les configurations, l'internationalisation et les routes dans les applications FastAPI",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/inicode_celestin03/pyarccore",
    packages=find_packages(),
    install_requires=[
        "fastapi>=0.68.0",
        "python-multipart>=0.0.5",
        "uvicorn>=0.15.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)